from datetime import date
import warnings

from ..core import ChineseNewYearCalendar
from ..registry_tools import iso_register
from ..exceptions import CalendarError

holidays = {
    2018:
        {
            'Ching Ming Festival': [(4, 5), (4, 6), (4, 7)],
            'Labour Day Holiday': [(4, 29), (4, 30), (5, 1)],
            'Dragon Boat Festival': [(6, 18)],
            'Mid-Autumn Festival': [(9, 24)],
            'New year': [(12, 30), (12, 31)]
        },
    2019:
        {
            'Ching Ming Festival': [(4, 5)],
            'Labour Day Holiday': [(5, 1), (5, 2), (5, 3)],
            'Dragon Boat Festival': [(6, 7)],
            'Mid-Autumn Festival': [(9, 13)]
        },
    2020:
        {
            'Ching Ming Festival': [(4, 4), (4, 5), (4, 6)],
            'Labour Day Holiday': [(5, 1), (5, 2), (5, 3), (5, 4), (5, 5)],
            'Dragon Boat Festival': [(6, 25), (6, 26), (6, 27)],
            'National Day': [(10, 8)]
        },
    2021:
        {
            'Ching Ming Festival': [(4, 3), (4, 4), (4, 5)],
            'Labour Day Holiday': [(5, 1), (5, 2), (5, 3), (5, 4), (5, 5)],
            'Dragon Boat Festival': [(6, 12), (6, 13), (6, 14)],
            'Mid-Autumn Festival': [(9, 19), (9, 20), (9, 21)]
        },
    2022:
        {
            'Ching Ming Festival': [(4, 3), (4, 4), (4, 5)],
            'Labour Day Holiday': [(4, 30), (5, 1), (5, 2), (5, 3), (5, 4)],
            'Dragon Boat Festival': [(6, 3), (6, 4), (6, 5)],
            'Mid-Autumn Festival': [(9, 10), (9, 11), (9, 12)]
        },
    2023:
        {
            'Ching Ming Festival': [(4, 5)],
            'Labour Day Holiday': [(4, 29), (4, 30), (5, 1), (5, 2), (5, 3)],
            'Dragon Boat Festival': [(6, 22), (6, 23), (6, 24)],
            'Mid-Autumn Festival': [(9, 29)],
            'National Day': [(9, 30)]
        },
    2024:
        {
            'Spring Festival': [(2, 16), (2, 17)],
            'Ching Ming Festival': [(4, 4), (4, 5), (4, 6)],
            'Labour Day Holiday': [(5, 1), (5, 2), (5, 3), (5, 4), (5, 5)],
            'Dragon Boat Festival': [(6, 8), (6, 9), (6, 10)],
            'Mid-Autumn Festival': [(9, 15), (9, 16), (9, 17)],
        },
    2025: {
        'New year': [(1, 1)],
        'Spring Festival': [(2, 4)],
        'Ching Ming Festival': [(4, 4), (4, 5), (4, 6)],
        'Labour Day Holiday': [(5, 1), (5, 2), (5, 3), (5, 4), (5, 5)],
        'Dragon Boat Festival': [(5, 31), (6, 1), (6, 2)],
        'National Day': [(10, 8)],
    }
}

workdays = {
    2018:
        {
            'Spring Festival Shift': [(2, 11), (2, 24)],
            'Ching Ming Festival Shift': [(4, 8)],
            'Labour Day Holiday Shift': [(4, 28)],
            'National Day Shift': [(9, 29), (9, 30)],
            'New year Shift': [(12, 29)]
        },
    2019:
        {
            'Spring Festival Shift': [(2, 2), (2, 3)],
            'Labour Day Holiday Shift': [(4, 28), (5, 5)],
            'National Day Shift': [(9, 29), (10, 12)],
        },
    2020:
        {
            'Spring Festival Shift': [(1, 19), (2, 1)],
            'Labour Day Holiday Shift': [(4, 26), (5, 9)],
            'Dragon Boat Festival Shift': [(6, 28)],
            'National Day Shift': [(9, 27), (10, 10)]
        },
    2021:
        {
            'Spring Festival Shift': [(2, 7), (2, 20)],
            'Labour Day Holiday Shift': [(4, 25), (5, 8)],
            'Mid-Autumn Festival Shift': [(9, 18)],
            'National Day Shift': [(9, 26), (10, 9)]
        },
    2022:
        {
            'Spring Festival Shift': [(1, 29), (1, 30)],
            'Ching Ming Festival Shift': [(4, 2)],
            'Labour Day Holiday Shift': [(4, 24), (5, 7)],
            'National Day Shift': [(10, 8), (10, 9)]
        },
    2023:
        {
            'Spring Festival Shift': [(1, 28), (1, 29)],
            'Labour Day Holiday Shift': [(4, 23), (5, 6)],
            'Dragon Boat Festival Shift': [(6, 25)],
            'National Day Shift': [(10, 7), (10, 8)]
        },
    2024:
        {
            'Spring Festival Shift': [(2, 4), (2, 9), (2, 18)],
            'Ching Ming Festival Shift': [(4, 7)],
            'Labour Day Holiday Shift': [(4, 28), (5, 11)],
            'Mid-Autumn Festival Shift': [(9, 14)],
            'National Day Shift': [(9, 29), (10, 12)]
        },
    2025:
        {
            'Spring Festival Shift': [(1, 26), (2, 8)],
            'Labour Day Holiday Shift': [(4, 27)],
            'National Day Shift': [(9, 28), (10, 11)],
        }
}


@iso_register('CN')
class China(ChineseNewYearCalendar):
    "China"
    # WARNING: Support 2018-2025 currently, need update every year.
    shift_new_years_day = True
    include_chinese_new_year_eve = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.extra_working_days = []
        for year, data in workdays.items():
            for working_day_name, day_list in data.items():
                for v in day_list:
                    self.extra_working_days.append(date(year, v[0], v[1]))

    def get_calendar_holidays(self, year):
        year_min, year_max = min(holidays), max(holidays)
        warnings.warn(
            f"Support years {year_min}-{year_max} currently, "
            f"need update every year."
        )
        if year not in holidays:
            msg = f"Need configure {year} for China."
            raise CalendarError(msg)
        return super().get_calendar_holidays(year)

    def get_variable_days(self, year):
        days = [(day, holiday_name) for (day, holiday_name)
                in super().get_variable_days(year)
                if day not in self.extra_working_days]

        # Spring Festival, eve, 1.1, and 1.2 - 1.6 in lunar day
        for i in range(2, 7):
            day = ChineseNewYearCalendar.lunar(year, 1, i)
            if day not in self.extra_working_days:
                days.append((day, "Spring Festival"))

        # National Days, 10.1 - 10.7 in general
        for i in range(1, 8):
            day = date(year, 10, i)
            if day not in self.extra_working_days:
                days.append((day, "National Day"))

        # other holidays
        for holiday_name, day_list in holidays[year].items():
            for v in day_list:
                days.append((date(year, v[0], v[1]), holiday_name))

        return days

    def is_working_day(self, day,
                       extra_working_days=None, extra_holidays=None):
        extra_working_days = extra_working_days or self.extra_working_days
        return super().is_working_day(day, extra_working_days, extra_holidays)

    def add_working_days(self, day, delta,
                         extra_working_days=None, extra_holidays=None,
                         keep_datetime=False):
        extra_working_days = extra_working_days or self.extra_working_days
        return super().add_working_days(day, delta,
                                        extra_working_days,
                                        extra_holidays,
                                        keep_datetime)

    def sub_working_days(self, day, delta,
                         extra_working_days=None, extra_holidays=None,
                         keep_datetime=False):
        extra_working_days = extra_working_days or self.extra_working_days
        return super().sub_working_days(day, delta,
                                        extra_working_days,
                                        extra_holidays,
                                        keep_datetime)



"""
Astronomical functions

Computed years spread from 30 years before and after the release year.
"""
import datetime
import gzip
import json
import pathlib
from typing import Tuple

TZAwareDate = datetime.date

# In 2021, Skyfield cannot compute ephemeris after date 2053-10-08 23:58:51Z UT
YEAR_INTERVAL = 30
TIME_ZONES = (
    'America/Santiago',
    'Asia/Hong_Kong',
    'Asia/Taipei',
    'Asia/Tokyo',
)
pre_computed_equinoxes_path = \
    pathlib.Path(__file__).parent / 'equinoxes.json.gz'
pre_computed_solar_terms_path = \
    pathlib.Path(__file__).parent / 'solar_terms.json.gz'

try:
    # Before Python 3.7, date.fromisoformat does not exist
    datetime.date.fromisoformat

    def fromisoformat(iso):
        return datetime.date.fromisoformat(iso)
except AttributeError:  # pre-3.7
    def fromisoformat(iso):
        try:
            if not iso or '-' not in iso:
                raise ValueError
            parts = [int(x) for x in iso.split('-')]
            if len(parts) != 3:
                raise ValueError
            return datetime.date(year=parts[0], month=parts[1], day=parts[2])
        except ValueError:
            raise ValueError(f"Invalid isoformat string: '{iso}'")


def create_astronomical_data(progress=None):
    from .skyfield_astronomy import (
        calculate_equinoxes as real_calculate_equinoxes
    )
    from .skyfield_astronomy import (
        solar_term as real_solar_term
    )
    if progress is None:
        progress = lambda it: it  # noqa: E731
    current_year = datetime.date.today().year
    first_year = current_year - YEAR_INTERVAL
    last_year = current_year + YEAR_INTERVAL
    years = range(first_year, last_year + 1)
    equinoxes = dict()
    solar_terms = dict()
    for i, time_zone in enumerate(TIME_ZONES, 1):
        print(f"{time_zone} {i}/{len(TIME_ZONES)}")
        equinoxes[time_zone] = dict()
        equinoxes_tz = equinoxes[time_zone]
        solar_terms[time_zone] = dict()
        solar_terms_tz = solar_terms[time_zone]
        for year in progress(years):
            equinoxes_tz[year] = tuple(
                equinox.isoformat()
                for equinox in real_calculate_equinoxes(year, time_zone)
            )
            solar_terms_tz[year] = dict()
            for degrees in range(15, 360, 15):
                solar_terms_tz[year][degrees] = \
                    real_solar_term(year, degrees, time_zone).isoformat()
    with gzip.open(pre_computed_equinoxes_path, 'wb') as f:
        f.write(json.dumps(equinoxes, ensure_ascii=False).encode('utf-8'))
    with gzip.open(pre_computed_solar_terms_path, 'wb') as f:
        f.write(json.dumps(solar_terms, ensure_ascii=False).encode('utf-8'))


def calculate_equinoxes(
    year: int,
    timezone: str = 'UTC',
) -> Tuple[TZAwareDate, TZAwareDate]:
    """
    calculate equinox with time zone.
    returns a 2-tuple with vernal and autumn equinoxes.
    """
    equinoxes = json.loads(gzip.decompress(
        pre_computed_equinoxes_path.read_bytes()
    ).decode('utf-8'))
    try:
        result = equinoxes[timezone][str(year)]
    except KeyError:
        raise NotImplementedError(
            f"The year {year} and timezone {timezone} are not pre-computed"
        )
    return tuple(fromisoformat(equinox) for equinox in result)


def solar_term(year: int, degrees: int, timezone: str = 'UTC') -> TZAwareDate:
    if not 15 <= degrees < 345 or degrees % 15 != 0:
        raise ValueError(
            "The degrees should be between 15 and 345 by step of 15"
        )
    solar_terms = json.loads(gzip.decompress(
        pre_computed_solar_terms_path.read_bytes()
    ).decode('utf-8'))
    try:
        solar_terms_per_tz_and_year = solar_terms[timezone][str(year)]
    except KeyError:
        raise NotImplementedError(
            f"The year {year} and timezone {timezone} are not pre-computed"
        )
    return fromisoformat(
        solar_terms_per_tz_and_year[str(degrees)]
    )



import datetime
import json
import pathlib
from unittest import TestCase
from unittest.mock import MagicMock, call, patch, sentinel

from freezegun import freeze_time

from .. import precomputed_astronomy as mod


class PreComputedAstronomyTest(TestCase):
    def test_year_interval(self):
        self.assertEqual(mod.YEAR_INTERVAL, 30)

    def test_time_zones(self):
        self.assertEqual(mod.TIME_ZONES, (
            'America/Santiago',
            'Asia/Hong_Kong',
            'Asia/Taipei',
            'Asia/Tokyo',
        ))

    def test_pre_computed_pathes(self):
        self.assertEqual(
            mod.pre_computed_equinoxes_path,
            pathlib.Path(__file__).parent.parent / 'equinoxes.json.gz',
        )
        self.assertEqual(
            mod.pre_computed_solar_terms_path,
            pathlib.Path(__file__).parent.parent / 'solar_terms.json.gz',
        )

    @patch('calendra.skyfield_astronomy.solar_term')
    @patch('calendra.skyfield_astronomy.calculate_equinoxes')
    @patch('calendra.precomputed_astronomy.gzip.open')
    @patch('calendra.precomputed_astronomy.YEAR_INTERVAL', 1)
    @patch('calendra.precomputed_astronomy.TIME_ZONES', ('Europe/Paris',))
    @freeze_time('2022-01-01')
    def test_create_astronomical_data(self,
                                      gzipopen,
                                      calculate_equinoxes,
                                      solar_term):
        equinoxes_file = MagicMock(name='equinoxes')
        equinoxes_file.__enter__.return_value = equinoxes_file
        solar_terms_file = MagicMock(name='solar_terms')
        solar_terms_file.__enter__.return_value = solar_terms_file
        gzipopen.side_effect = [equinoxes_file, solar_terms_file]
        calculate_equinoxes.side_effect = lambda year, timezone: (
            datetime.date(year=year, month=2, day=15),
            datetime.date(year=year, month=3, day=16),
        )
        solar_term.side_effect = lambda year, degrees, timezone: \
            datetime.date(year=year, month=7, day=21)
        mod.create_astronomical_data()
        gzipopen.assert_has_calls([
            call(mod.pre_computed_equinoxes_path, 'wb'),
            call(mod.pre_computed_solar_terms_path, 'wb'),
        ], any_order=True)
        expected_equinoxes_dict = {
            'Europe/Paris': {
                '2021': ('2021-02-15', '2021-03-16'),
                '2022': ('2022-02-15', '2022-03-16'),
                '2023': ('2023-02-15', '2023-03-16'),
            }
        }
        equinoxes_file.write.assert_called_once_with(
            json.dumps(expected_equinoxes_dict).encode('utf-8')
        )
        expected_solar_terms_dict = {
            'Europe/Paris': {
                '2021': {str(i): '2021-07-21' for i in range(15, 360, 15)},
                '2022': {str(i): '2022-07-21' for i in range(15, 360, 15)},
                '2023': {str(i): '2023-07-21' for i in range(15, 360, 15)},
            }
        }
        solar_terms_file.write.assert_called_once_with(
            json.dumps(expected_solar_terms_dict).encode('utf-8')
        )

    @patch('calendra.precomputed_astronomy.gzip.decompress')
    @patch('calendra.precomputed_astronomy.pre_computed_equinoxes_path')
    def test_calculate_equinoxes(self,
                                 pre_computed_equinoxes_path,
                                 decompress):
        pre_computed_equinoxes_path.read_bytes.return_value = \
            sentinel.some_equinoxes_bytes
        decompress.return_value = json.dumps({
            'Europe/Paris': {
                '2021': ('2021-02-15', '2021-03-16'),
                '2022': ('2022-02-15', '2022-03-16'),
                '2023': ('2023-02-15', '2023-03-16'),
            }
        }).encode('utf-8')
        with self.assertRaises(NotImplementedError):
            mod.calculate_equinoxes(2100, 'Europe/Paris')
        pre_computed_equinoxes_path.read_bytes.assert_called_once()
        pre_computed_equinoxes_path.reset_mock()
        decompress.assert_called_once_with(sentinel.some_equinoxes_bytes)
        decompress.reset_mock()
        with self.assertRaises(NotImplementedError):
            mod.calculate_equinoxes(2022, 'Europe/Berlin')
        pre_computed_equinoxes_path.read_bytes.assert_called_once()
        pre_computed_equinoxes_path.reset_mock()
        decompress.assert_called_once_with(sentinel.some_equinoxes_bytes)
        decompress.reset_mock()
        self.assertEqual(mod.calculate_equinoxes(2022, 'Europe/Paris'), (
            datetime.date(year=2022, month=2, day=15),
            datetime.date(year=2022, month=3, day=16),
        ))
        pre_computed_equinoxes_path.read_bytes.assert_called_once()
        pre_computed_equinoxes_path.reset_mock()
        decompress.assert_called_once_with(sentinel.some_equinoxes_bytes)
        decompress.reset_mock()

    @patch('calendra.precomputed_astronomy.gzip.decompress')
    @patch('calendra.precomputed_astronomy.pre_computed_solar_terms_path')
    def test_sorted_term(self,
                         pre_computed_solar_terms_path,
                         decompress):
        pre_computed_solar_terms_path.read_bytes.return_value = \
            sentinel.some_solar_terms_bytes
        decompress.return_value = json.dumps({
            'Europe/Paris': {
                '2021': {str(i): '2021-07-21' for i in range(15, 360, 15)},
                '2022': {str(i): '2022-07-21' for i in range(15, 360, 15)},
                '2023': {str(i): '2023-07-21' for i in range(15, 360, 15)},
            }
        }).encode('utf-8')
        with self.assertRaises(ValueError):
            mod.solar_term(2022, 0, 'Europe/Paris')
        with self.assertRaises(ValueError):
            mod.solar_term(2022, 360, 'Europe/Paris')
        with self.assertRaises(ValueError):
            mod.solar_term(2022, 20, 'Europe/Paris')
        with self.assertRaises(NotImplementedError):
            mod.solar_term(2100, 45, 'Europe/Paris')
        pre_computed_solar_terms_path.read_bytes.assert_called_once()
        pre_computed_solar_terms_path.reset_mock()
        decompress.assert_called_once_with(sentinel.some_solar_terms_bytes)
        decompress.reset_mock()
        with self.assertRaises(NotImplementedError):
            mod.solar_term(2022, 90, 'Europe/Berlin')
        pre_computed_solar_terms_path.read_bytes.assert_called_once()
        pre_computed_solar_terms_path.reset_mock()
        decompress.assert_called_once_with(sentinel.some_solar_terms_bytes)
        decompress.reset_mock()
        self.assertEqual(
            mod.solar_term(2022, 90, 'Europe/Paris'),
            datetime.date(year=2022, month=7, day=21),
        )
        pre_computed_solar_terms_path.read_bytes.assert_called_once()
        pre_computed_solar_terms_path.reset_mock()
        decompress.assert_called_once_with(sentinel.some_solar_terms_bytes)
        decompress.reset_mock()



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('FR')
class France(WesternCalendar):
    'France'

    # Christian holidays
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_all_saints = True
    include_assumption = True

    # Civil holidays
    include_labour_day = True
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 8, "Victory in Europe Day"),
        (7, 14, "Bastille Day"),
        (11, 11, "Armistice Day"),
    )


class FranceAlsaceMoselle(France):
    "France Alsace/Moselle"
    include_good_friday = True
    include_boxing_day = True



from ..core import OrthodoxCalendar
from ..registry_tools import iso_register


@iso_register('GE')
class Georgia(OrthodoxCalendar):
    'Country of Georgia'

    "Sources: "
    "https://en.wikipedia.org/wiki/Public_holidays_in_Georgia_(country)"
    "https://www.officeholidays.com/countries/georgia/2021"

    include_christmas = False
    include_christmas_eve = False

    include_new_years_day = True
    include_orthodox_christmas = True
    include_epiphany = False
    include_good_friday = True
    include_easter_saturday = True
    include_easter_sunday = True
    include_easter_monday = True

    FIXED_HOLIDAYS = OrthodoxCalendar.FIXED_HOLIDAYS + (
        (1, 2, "Day After New Year"),
        (1, 19, "Orthodox Epiphany"),
        (3, 3, "Mother's Day"),
        (3, 8, "International Women's Day"),
        (4, 9, "Day Of National Unity"),
        (5, 9, "Day Of Victory Over Fascism"),
        (5, 12, "Saint Andrew The First-Called Day"),
        (5, 26, "Independence Day"),
        (8, 28, "Saint Mary's Day"),
        (10, 14, "Day Of Svetitskovloba"),
        (11, 23, "Saint George's Day"),
    )



from ..registry_tools import iso_register
from .core import UnitedStates


@iso_register('US-NY')
class NewYork(UnitedStates):
    """New York"""
    include_lincoln_birthday = True
    include_election_day_every_year = True



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('IT')
class Italy(WesternCalendar):
    'Italy'

    # Civil holidays
    include_labour_day = True
    labour_day_label = "International Workers' Day"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (4, 25, "Liberation Day"),
        (6, 2, "Republic Day"),
    )

    # Christian holidays
    include_immaculate_conception = True
    include_epiphany = True
    include_easter_monday = True
    include_assumption = True
    include_all_saints = True
    include_boxing_day = True
    boxing_day_label = "St Stephen's Day"
    observance_shift = None



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('AT')
class Austria(WesternCalendar):
    'Austria'

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (10, 26, "National Holiday"),  # Nationalfeiertag
    )
    # Civil holidays
    include_labour_day = True
    labour_day_label = "State Holiday"  # Staatsfeiertag
    # Christian holidays
    include_epiphany = True
    include_easter_monday = True
    include_ascension = True
    include_whit_monday = True
    include_corpus_christi = True
    include_assumption = True
    include_all_saints = True
    include_immaculate_conception = True
    include_christmas = True
    include_boxing_day = True
    boxing_day_label = "St. Stephen's Day"  # Stefanitag



from datetime import date

from . import GenericCalendarTest
from ..europe import (
    Aargau, AppenzellInnerrhoden, AppenzellAusserrhoden, Bern, BaselLandschaft,
    BaselStadt, Fribourg, Geneva, Glarus, Graubunden, Jura, Luzern, Neuchatel,
    Nidwalden, Obwalden, StGallen, Schaffhausen, Solothurn, Schwyz, Thurgau,
    Ticino, Uri, Vaud, Valais, Zug, Zurich
)


class AargauTest(GenericCalendarTest):
    cal_class = Aargau

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class AppenzellInnerrhodenTest(GenericCalendarTest):
    cal_class = AppenzellInnerrhoden

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class AppenzellAusserrhodenTest(GenericCalendarTest):
    cal_class = AppenzellAusserrhoden

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class BernTest(GenericCalendarTest):
    cal_class = Bern

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class BaselLandschaftTest(GenericCalendarTest):
    cal_class = BaselLandschaft

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class BaselStadtTest(GenericCalendarTest):
    cal_class = BaselStadt

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class FribourgTest(GenericCalendarTest):
    cal_class = Fribourg

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class GenevaTest(GenericCalendarTest):
    cal_class = Geneva

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        # Jeune Genevois, only in Geneva
        self.assertIn(date(2020, 9, 10), holidays)
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        # Restoration day - Geneva
        self.assertIn(date(2020, 12, 31), holidays)

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        # Jeune Genevois, only in Geneva
        self.assertIn(date(2021, 9, 9), holidays)
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        # Restoration day - Geneva
        self.assertIn(date(2021, 12, 31), holidays)


class GlarusTest(GenericCalendarTest):
    cal_class = Glarus

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        # Only in Glarus/Glaris
        self.assertIn(date(2020, 4, 3), holidays)   # Näfels Day
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        # Only in Glarus/Glaris
        self.assertIn(date(2021, 4, 3), holidays)   # Näfels Day
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class GraubundenTest(GenericCalendarTest):
    cal_class = Graubunden

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2020, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2021, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class JuraTest(GenericCalendarTest):
    cal_class = Jura

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        # Independance day - Only in Jura
        self.assertIn(date(2020, 6, 23), holidays)
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 25), holidays)  # XMas

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        # Independance day - Only in Jura
        self.assertIn(date(2021, 6, 23), holidays)
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 25), holidays)  # XMas


class LuzernTest(GenericCalendarTest):
    cal_class = Luzern

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2020, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2021, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class NeuchatelTest(GenericCalendarTest):
    cal_class = Neuchatel

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertNotIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 3, 1), holidays)  # Republic Day
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        # Lundi du Jeûne - Only in Vaud/Neuchatel
        self.assertIn(date(2020, 9, 21), holidays)
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertNotIn(date(2022, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertNotIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 3, 1), holidays)  # Republic Day
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        # Lundi du Jeûne - Only in Vaud/Neuchatel
        self.assertIn(date(2021, 9, 20), holidays)
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertNotIn(date(2022, 12, 26), holidays)  # St Stephen's day

    def test_year_2022(self):
        holidays = self.cal.holidays_set(2022)
        # Only when Christmas is on Sunday
        self.assertIn(date(2022, 12, 26), holidays)  # St Stephen's day

    def test_year_2023(self):
        holidays = self.cal.holidays_set(2023)
        # Only when New year is on Sunday
        self.assertIn(date(2023, 1, 2), holidays)  # Berchtolds


class NidwaldenTest(GenericCalendarTest):
    cal_class = Nidwalden

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class ObwaldenTest(GenericCalendarTest):
    cal_class = Obwalden

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 9, 25), holidays)  # Saint Nicholas of Flüe
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 9, 25), holidays)  # Saint Nicholas of Flüe
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class StGallenTest(GenericCalendarTest):
    cal_class = StGallen

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class SchaffhausenTest(GenericCalendarTest):
    cal_class = Schaffhausen

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class SolothurnTest(GenericCalendarTest):
    cal_class = Solothurn

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class SchwyzTest(GenericCalendarTest):
    cal_class = Schwyz

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2020, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2021, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class ThurgauTest(GenericCalendarTest):
    cal_class = Thurgau

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class TicinoTest(GenericCalendarTest):
    cal_class = Ticino

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2020, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        # St Peter & St Paul - Only in Ticino
        self.assertIn(date(2020, 6, 29), holidays)
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2021, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        # St Peter & St Paul - Only in Ticino
        self.assertIn(date(2021, 6, 29), holidays)
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class UriTest(GenericCalendarTest):
    cal_class = Uri

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2020, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 6), holidays)  # Epiphany
        self.assertIn(date(2021, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class VaudTest(GenericCalendarTest):
    cal_class = Vaud

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        # Lundi du Jeûne - Only in Vaud/Neuchatel
        self.assertIn(date(2020, 9, 21), holidays)
        self.assertIn(date(2020, 12, 25), holidays)  # XMas

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        # Lundi du Jeûne - Only in Vaud/Neuchatel
        self.assertIn(date(2021, 9, 20), holidays)
        self.assertIn(date(2021, 12, 25), holidays)  # XMas


class ValaisTest(GenericCalendarTest):
    cal_class = Valais

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 3, 19), holidays)  # St Joseph
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas


class ZugTest(GenericCalendarTest):
    cal_class = Zug

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 6, 11), holidays)  # Corpus Christi
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 8, 15), holidays)  # Assumption
        self.assertIn(date(2020, 11, 1), holidays)  # All Saints
        self.assertIn(date(2020, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 6, 3), holidays)  # Corpus Christi
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 8, 15), holidays)  # Assumption
        self.assertIn(date(2021, 11, 1), holidays)  # All Saints
        self.assertIn(date(2021, 12, 8), holidays)  # Immaculate conception
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day


class ZurichTest(GenericCalendarTest):
    cal_class = Zurich

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New years day
        self.assertIn(date(2020, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 21), holidays)  # Ascension day
        self.assertIn(date(2020, 6, 1), holidays)  # Whit Monday
        self.assertIn(date(2020, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2020, 12, 25), holidays)  # XMas
        self.assertIn(date(2020, 12, 26), holidays)  # St Stephen's day

    def test_year_2021(self):
        holidays = self.cal.holidays_set(2021)
        self.assertIn(date(2021, 1, 1), holidays)  # New years day
        self.assertIn(date(2021, 1, 2), holidays)  # Berchtolds
        self.assertIn(date(2021, 4, 2), holidays)  # Good Friday
        self.assertIn(date(2021, 4, 5), holidays)  # Easter Monday
        self.assertIn(date(2021, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2021, 5, 13), holidays)  # Ascension day
        self.assertIn(date(2021, 5, 24), holidays)  # Whit Monday
        self.assertIn(date(2021, 8, 1), holidays)  # Swiss National Day
        self.assertIn(date(2021, 12, 25), holidays)  # XMas
        self.assertIn(date(2021, 12, 26), holidays)  # St Stephen's day



from .core import UnitedStates
from ..core import MON
from ..registry_tools import iso_register


@iso_register('US-AL')
class Alabama(UnitedStates):
    "Alabama"
    include_confederation_day = True
    martin_luther_king_label = "Robert E. Lee/Martin Luther King Birthday"
    presidents_day_label = "George Washington/Thomas Jefferson Birthday"
    columbus_day_label = ("Columbus Day / Fraternal Day /"
                          " American Indian Heritage Day")
    include_jefferson_davis_birthday = True


class AlabamaBaldwinCounty(Alabama):
    "Baldwin County, Alabama"
    include_fat_tuesday = True


class AlabamaMobileCounty(Alabama):
    "Mobile County, Alabama"
    include_fat_tuesday = True


class AlabamaPerryCounty(Alabama):
    "Mobile Perry, Alabama"

    def get_obama_day(self, year):
        """
        Obama Day happens on the 2nd MON of November.
        """
        return (
            self.get_nth_weekday_in_month(year, 11, MON, 2),
            "Obama Day"
        )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append(self.get_obama_day(year))
        return days



from datetime import date

from ..core import IslamicCalendar, SAT, SUN
from ..registry_tools import iso_register


@iso_register('TN')
class Tunisia(IslamicCalendar):
    "Tunisia"
    # Civil holidays
    include_labour_day = True
    # Islamic holidays
    include_prophet_birthday = True
    include_eid_al_fitr = True
    length_eid_al_fitr = 2
    include_day_of_sacrifice = True
    length_eid_al_adha = 4
    include_islamic_new_year = True

    FIXED_HOLIDAYS = IslamicCalendar.FIXED_HOLIDAYS + (
            (3, 20, "Independence Day"),
            (7, 25, "Republic Day"),
            (4, 9, "Martyrs' Day"),
            (8, 13, "Women's Day"),
            (10, 15, "Evacuation Day")
        )

    # Tunisia has adopted the "western" workweek.
    WEEKEND_DAYS = (SAT, SUN)

    def get_fixed_holidays(self, year):
        """Get fixed holidays.

        Args:
            year (int): Year

        Returns:
            tuple: Tuple of date and label
        """
        days = super().get_fixed_holidays(year)

        revolution_day_label = "Revolution Day"

        if 2011 < year <= 2020:
            days.append((date(year, 1, 14), revolution_day_label))

        if year >= 2021:
            days.append((date(year, 12, 17), revolution_day_label))

        return days



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('SV')
class ElSalvador(WesternCalendar):
    "El Salvador"
    # Civil holidays
    include_labour_day = True
    # Christian holidays
    include_holy_thursday = True
    include_good_friday = True
    include_easter_saturday = True

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 10, "Mothers' Day"),
        (6, 17, "Fathers' Day"),
        (8, 6, "Celebrations of San Salvador"),
        (9, 15, "Independence Day"),
        (11, 2, "All Saints Day"),
    )



from unittest import TestCase

from ..registry import registry


class GlobalRegistry(TestCase):

    def test_name(self):
        for klass in registry.region_registry.values():
            # All classes have a `name` class property
            self.assertTrue(hasattr(klass, 'name'))
            # All classes have a non-empty name
            self.assertTrue(klass.name)
            # All those properties are equivalent to the class docstring
            self.assertEqual(klass.name, klass.__doc__)



"""
Tools to update the ISO registry.
"""


def iso_register(iso_code):
    """
    Registers Calendar class as country or region in IsoRegistry.

    Registered country must set class variables ``iso`` using this decorator.

    >>> from calendra.core import Calendar
    >>> from calendra.registry import registry
    >>> from calendra.registry_tools import iso_register
    >>> @iso_register('MC-MR')
    ... class MyRegion(Calendar):
    ...     'My Region'

    Region calendar is then retrievable from registry:

    >>> calendar = registry.get('MC-MR')
    """

    def wrapper(cls):
        from calendra.registry import registry
        registry.register(iso_code, cls)
        return cls
    return wrapper
